@@ -1,44 +1,75 @@
 #
 #     ewcdf.R
 #
-#     $Revision: 1.12 $  $Date: 2017/08/30 02:04:46 $
+#     $Revision: 1.18 $  $Date: 2019/03/12 11:14:36 $
 #
 #  With contributions from Kevin Ummel
 #
 
-ewcdf <- function(x, weights=rep(1/length(x), length(x)))
+ewcdf <- function(x, weights=NULL, normalise=TRUE, adjust=1)
 {
-  nw <- length(weights)
   nx <- length(x)
-  if(nw == 0) {
-    weights <- rep(1/nx, nx)
-  } else if(nw == 1) {
-    weights <- rep(weights, nx)
-  } else if(nw != nx) stopifnot(length(x) == length(weights))
-  # remove NA's together
+  nw <- length(weights)
+  weighted <- (nw > 0)
+
+  if(weighted) {
+    check.nvector(weights, things="entries of x", oneok=TRUE)
+    stopifnot(all(weights >= 0))
+    if(nw == 1) 
+      weights <- rep(weights, nx)
+  }
+  
+  ## remove NA's
   nbg <- is.na(x) 
   x <- x[!nbg]
-  weights <- weights[!nbg]
+  if(weighted) weights <- weights[!nbg]
   n <- length(x)
   if (n < 1)
     stop("'x' must have 1 or more non-missing values")
-  stopifnot(all(weights >= 0))
-  # sort in increasing order of x value
-  ox <- fave.order(x)
-  x <- x[ox]
-  w <- weights[ox]
-  # find jump locations and match
-  vals <- sort(unique(x))
-  xmatch <- factor(match(x, vals), levels=seq_along(vals))
-  # sum weight in each interval
-  wmatch <- tapply(w, xmatch, sum)
-  wmatch[is.na(wmatch)] <- 0
+
+  ## sort in increasing order of x value
+  if(!weighted) {
+    x <- sort(x)
+    w <- rep(1, n)
+  } else {
+    ox <- fave.order(x)
+    x <- x[ox]
+    w <- weights[ox]
+  }
+  ## find jump locations and match
+  rl <- rle(x)
+  vals <- rl$values
+  if(!weighted) {
+    wmatch <- rl$lengths
+  } else {
+    nv <- length(vals)
+    wmatch <- .C("tabsumweight",
+                 nx=as.integer(n),
+                 x=as.double(x),
+                 w=as.double(w),
+                 nv=as.integer(nv),
+                 v=as.double(vals),
+                 z=as.double(numeric(nv)),
+                 PACKAGE="spatstat")$z
+  }
+  ## cumulative weight in each interval
   cumwt <- cumsum(wmatch)
-  # make function
+  totwt <- sum(wmatch)
+  ## rescale ?
+  if(normalise) {
+    cumwt <- cumwt/totwt
+    totwt <- 1
+  } else if(adjust != 1) {
+    cumwt <- adjust * cumwt
+    totwt <- adjust * totwt
+  }
+  ## make function
   rval <- approxfun(vals, cumwt,
-                    method = "constant", yleft = 0, yright = sum(wmatch),
+                    method = "constant", yleft = 0, yright = totwt,
                     f = 0, ties = "ordered")
-  class(rval) <- c("ewcdf", "ecdf", "stepfun", class(rval))
+  class(rval) <- c("ewcdf",
+                   if(normalise) "ecdf" else NULL,
+                   "stepfun", class(rval))
   assign("w", w, envir=environment(rval))
   attr(rval, "call") <- sys.call()
   return(rval)
