@@ -2,7 +2,7 @@
 #
 #   disc.R
 #
-#   $Revision: 1.27 $ $Date: 2017/06/05 10:31:58 $
+#   $Revision: 1.29 $ $Date: 2019/03/04 04:32:29 $
 #
 #   Compute the disc of radius r in a linear network
 #
@@ -143,12 +143,18 @@ lineardisc <- function(L, x=locator(1), r, plotit=TRUE,
   return(list(lines=disclines, endpoints=discends))
 }
 
-countends <- function(L, x=locator(1), r, toler=NULL) {
+countends <- function(L, x=locator(1), r, toler=NULL, internal=list()) {
   # L is the linear network (object of class "linnet")
   # x is the centre point of the disc
   # r is the radius of the disc
   #
   stopifnot(inherits(L, "linnet"))
+  sparse <- L$sparse %orifnull% is.null(L$dpath)
+  if(sparse)
+    stop(paste("countends() does not support linear networks",
+               "that are stored in sparse matrix format.",
+               "Please convert the data using as.linnet(sparse=FALSE)"),
+         call.=FALSE)
   # get x
   if(missing(x))
     x <- clickppp(1, Window(L), add=TRUE)
@@ -158,11 +164,13 @@ countends <- function(L, x=locator(1), r, toler=NULL) {
   
   if(length(r) != np)
     stop("Length of vector r does not match number of points in x")
-  #
-  if(!is.connected(L)) {
+  
+  ## determine whether network is connected
+  iscon <- internal$is.connected %orifnull% is.connected(L)
+  if(!iscon) {
     #' disconnected network - split into components
     result <- numeric(np)
-    lab <- connected(L, what="labels")
+    lab <- internal$connected.labels %orifnull% connected(L, what="labels")
     subsets <- split(seq_len(nvertices(L)), factor(lab))
     for(subi in subsets) {
       xi <- thinNetwork(x, retainvertices=subi)
@@ -170,7 +178,8 @@ countends <- function(L, x=locator(1), r, toler=NULL) {
       ok <- is.finite(r[witch])
       witchok <- witch[ok]
       result[witchok] <-
-        countends(domain(xi), xi[ok], r[witchok], toler=toler)      
+        countends(domain(xi), xi[ok], r[witchok], toler=toler,
+                  internal=list(is.connected=TRUE))      
     }
     return(result)
   }
