@@ -3,7 +3,7 @@
 #    
 #    Linear networks
 #
-#    $Revision: 1.66 $    $Date: 2018/03/07 02:20:39 $
+#    $Revision: 1.68 $    $Date: 2019/01/31 08:40:24 $
 #
 # An object of class 'linnet' defines a linear network.
 # It includes the following components
@@ -59,7 +59,7 @@ linnet <- function(vertices, m, edges, sparse=FALSE, warn=TRUE) {
     from <- ij[,1L]
     to   <- ij[,2L]
   } else {
-    # check (from, to) pairs
+    ## check (from, to) pairs
     stopifnot(is.matrix(edges) && ncol(edges) == 2)
     if(any((edges %% 1) != 0))
       stop("Entries of edges list should be integers")
@@ -72,7 +72,16 @@ linnet <- function(vertices, m, edges, sparse=FALSE, warn=TRUE) {
       stop("index out-of-bounds in edges list")
     from <- edges[,1L]
     to   <- edges[,2L]
-    # convert to adjacency matrix
+    ## avoid duplication in either sense
+    up <- (from < to)
+    ee <- cbind(ifelse(up, from , to), ifelse(up, to, from))
+    if(anyDuplicated(ee)) {
+      warning("Duplicated segments were ignored", call.=FALSE)
+      ok <- !duplicated(ee)
+      from <- from[ok]
+      to   <- to[ok]
+    }
+    ## convert to adjacency matrix
     if(!sparse) {
       m <- matrix(FALSE, np, np)
       m[edges] <- TRUE
@@ -277,39 +286,31 @@ as.linnet.linnet <- function(X, ..., sparse) {
   return(X)
 }
 
-as.linnet.psp <- local({
-  
-  as.linnet.psp <- function(X, ..., eps, sparse=FALSE) {
-    X <- selfcut.psp(X)
-    V <- unique(endpoints.psp(X))
-    if(missing(eps) || is.null(eps)) {
-      eps <- sqrt(.Machine$double.eps) * diameter(Frame(X))
-    } else {
-      check.1.real(eps)
-      stopifnot(eps >= 0)
-    }
-    if(eps > 0 && minnndist(V) <= eps) {
-      gV <- marks(connected(V, eps))
-      xy <- split(coords(V), gV)
-      mxy <- lapply(xy, centro)
-      V <- do.call(superimpose, append(unname(mxy), list(W=Window(X))))
-    }
-    first  <- endpoints.psp(X, "first")
-    second <- endpoints.psp(X, "second")
-    from <- nncross(first, V, what="which")
-    to   <- nncross(second, V, what="which")
-    nontrivial <- (from != to)
-    join <- cbind(from, to)[nontrivial, , drop=FALSE]
-    result <- linnet(V, edges=join, sparse=sparse)
-    if(is.marked(X)) marks(result$lines) <- marks(X[nontrivial])
-    return(result)
+as.linnet.psp <- function(X, ..., eps, sparse=FALSE) {
+  X <- selfcut.psp(X)
+  V <- unique(endpoints.psp(X))
+  if(missing(eps) || is.null(eps)) {
+    eps <- sqrt(.Machine$double.eps) * diameter(Frame(X))
+  } else {
+    check.1.real(eps)
+    stopifnot(eps >= 0)
   }
-
-  centro <- function(X) as.list(apply(X, 2, mean))
-  
-  as.linnet.psp
-})
-
+  if(eps > 0 && minnndist(V) <= eps) {
+    gV <- marks(connected(V, eps))
+    xx <- as.numeric(by(V$x, gV, mean))
+    yy <- as.numeric(by(V$y, gV, mean))
+    V <- ppp(xx, yy, window=Window(X))
+  }
+  first  <- endpoints.psp(X, "first")
+  second <- endpoints.psp(X, "second")
+  from <- nncross(first, V, what="which")
+  to   <- nncross(second, V, what="which")
+  nontrivial <- (from != to)
+  join <- cbind(from, to)[nontrivial, , drop=FALSE]
+  result <- linnet(V, edges=join, sparse=sparse)
+  if(is.marked(X)) marks(result$lines) <- marks(X[nontrivial])
+  return(result)
+}
 
 unitname.linnet <- function(x) {
   unitname(x$window)
