@@ -3,7 +3,7 @@
 #'
 #'   Tessellations on a Linear Network
 #'
-#'   $Revision: 1.16 $   $Date: 2017/12/18 00:57:05 $
+#'   $Revision: 1.30 $   $Date: 2019/02/07 04:45:47 $
 #'
 
 lintess <- function(L, df) {
@@ -169,24 +169,96 @@ Window.lintess <- function(X, ...) { as.owin(as.linnet(X)) }
 
 domain.lintess <- as.linnet.lintess <- function(X, ...) { X$L }
 
+lineartileindex <- function(seg, tp, Z,
+                            method=c("encode", "C", "interpreted")) {
+  method <- match.arg(method)
+  df <- if(inherits(Z, "lintess")) Z$df else
+        if(is.data.frame(Z)) Z else stop("Format of Z is unrecognised")
+  switch(method,
+         interpreted = {
+           n <- length(seg)
+           #' extract tessellation data
+           tilenames <- levels(df$tile)
+           answer <- factor(rep(NA_integer_, n),
+                            levels=seq_along(tilenames),
+                            labels=tilenames)
+           for(i in seq_along(seg)) {
+             tpi <- tp[i]
+             segi <- seg[i]
+             j <- which(df$seg == segi)
+             kk <- which(df[j, "t0"] <= tpi & df[j, "t1"] >= tpi)
+             answer[i] <- if(length(kk) == 0) NA else df[j[min(kk)], "tile"]
+           }
+         },
+         encode = {
+           #' encode locations as numeric
+           loc <- seg - 1 + tp
+           #' extract tessellation data and sort them
+           df <- df[order(df$seg, df$t0), , drop=FALSE]
+           m <- nrow(df)
+           #' encode breakpoints as numeric
+           bks <- with(df, c(seg - 1 + t0, seg[m]))
+           #' which piece contains each query point
+           jj <- findInterval(loc, bks,
+                              left.open=TRUE, all.inside=TRUE,
+                              rightmost.closed=TRUE)
+           answer <- df$tile[jj]
+         },
+         C = {
+           #' sort query data
+           oo <- order(seg, tp)
+           seg <- seg[oo]
+           tp  <- tp[oo]
+           n <- length(seg)
+           #' extract tessellation data and sort them
+           df <- df[order(df$seg, df$t0), , drop=FALSE]
+           m <- nrow(df)
+           #' handle factor 
+           dftile <- df$tile
+           tilecode <- as.integer(dftile)
+           tilenames <- levels(dftile)
+           #' launch
+           z <- .C("lintileindex",
+                   n      = as.integer(n),
+                   seg    = as.integer(seg),
+                   tp     = as.double(tp),
+                   dfn    = as.integer(m),
+                   dfseg  = as.integer(df$seg),
+                   dft0   = as.double(df$t0),
+                   dft1   = as.double(df$t1),
+                   dftile = as.integer(tilecode),
+                   answer = as.integer(integer(n)),
+                   PACKAGE="spatstat")
+           z <- z$answer
+           z[z == 0] <- NA
+           answer <- integer(n)
+           answer[oo] <- z
+           answer <- factor(answer,
+                            levels=seq_along(tilenames),
+                            labels=tilenames)
+           })
+  return(answer)
+}
+
 as.linfun.lintess <- function(X, ..., values, navalue=NA) {
   L <- X$L
-  df <- X$df
+  Xdf <- X$df
+  tilenames <- levels(Xdf$tile)
   if(missing(values) || is.null(values)) {
-    rowvalues <- df$tile
+    tilevalues <- factor(tilenames, levels=tilenames)
   } else {
-    if(length(values) != length(levels(df$tile)))
+    if(length(values) != length(tilenames))
       stop("Length of 'values' should equal the number of tiles", call.=FALSE)
-    rowvalues <- values[as.integer(df$tile)]    
+    tilevalues <- values
   }
   f <- function(x, y, seg, tp) {
-    result <- rowvalues[integer(0)]
-    for(i in seq_along(seg)) {
-      tpi <- tp[i]
-      segi <- seg[i]
-      j <- which(df$seg == segi)
-      kk <- which(df[j, "t0"] <= tpi & df[j, "t1"] >= tpi)
-      result[i] <- if(length(kk) == 0) navalue else rowvalues[j[min(kk)]]
+    k <- as.integer(lineartileindex(seg, tp, Xdf))
+    if(!anyNA(k)) {
+      result <- tilevalues[k]
+    } else {
+      ok <- !is.na(k)
+      result <- rep(navalue, length(seg))
+      result[ok] <- tilevalues[k[ok]]
     }
     return(result)
   }
