@@ -3,7 +3,7 @@
 #
 # code to plot transformation diagnostic
 #
-#   $Revision: 1.8 $  $Date: 2016/07/15 10:22:03 $
+#   $Revision: 1.11 $  $Date: 2019/03/14 08:04:11 $
 #
 
 parres <- function(model, covariate, ...,
@@ -13,10 +13,17 @@ parres <- function(model, covariate, ...,
                    bw.restrict = FALSE,
                    covname) {  
 
+  callstring <- paste(deparse(sys.call()), collapse = "")
   modelname <- deparse(substitute(model))
+
+  stopifnot(is.ppm(model))
+
+  if(missing(covariate)) {
+    mc <- model.covariates(model)
+    if(length(mc) == 1) covariate <- mc else stop("covariate must be provided")
+  }
   if(missing(covname)) 
     covname <- sensiblevarname(deparse(substitute(covariate)), "X")
-  callstring <- paste(deparse(sys.call()), collapse = "")
 
   if(is.marked(model))
     stop("Sorry, this is not yet implemented for marked models")
@@ -29,7 +36,6 @@ parres <- function(model, covariate, ...,
   bw.input <- match.arg(bw.input)
   
   # validate model
-  stopifnot(is.ppm(model))
   modelcall <- model$callstring
   if(is.null(modelcall))
     modelcall <- model$call
@@ -166,14 +172,14 @@ parres <- function(model, covariate, ...,
       # check whether covariate is separable
       if(any(conflict <- dmat[relevant, othercov, drop=FALSE])) {
         ## identify entangled covariates
-        entangled <- colnames(conflict)[apply(conflict, 2, any)]
+        entangled <- colnames(conflict)[matcolany(conflict)]
         ## not problematic if constant
         ok <- unlist(isconstant[entangled])
         conflict[ , ok] <- FALSE
         ## re-test
         if(any(conflict)) {
-          conflictterms <- apply(conflict, 1, any)
-          conflictcovs  <- apply(conflict, 2, any)
+          conflictterms <- matrowany(conflict)
+          conflictcovs  <- matcolany(conflict)
           stop(paste("The covariate", sQuote(covname),
                      "cannot be separated from the",
                      ngettext(sum(conflictcovs), "covariate", "covariates"),
@@ -238,14 +244,14 @@ parres <- function(model, covariate, ...,
       # check whether covariate is separable
       if(any(conflict<- offmat[relevant, othercov, drop=FALSE])) {
         ## identify entangled covariates
-        entangled <- colnames(conflict)[apply(conflict, 2, any)]
+        entangled <- colnames(conflict)[matcolany(conflict)]
         ## not problematic if constant
         ok <- unlist(isconstant[entangled])
         conflict[ , ok] <- FALSE
         ## re-test
         if(any(conflict)) {
-          conflictterms <- apply(conflict, 1, any)
-          conflictcovs  <- apply(conflict, 2, any)
+          conflictterms <- matrowany(conflict)
+          conflictcovs  <- matcolany(conflict)
           stop(paste("The covariate", sQuote(covname),
                      "cannot be separated from the",
                      ngettext(sum(conflictcovs), "covariate", "covariates"),
@@ -271,7 +277,7 @@ parres <- function(model, covariate, ...,
                    offnames[nbg],
                    "not found in model frame"))
       effex <- mf[, offnames, drop=FALSE]
-      effect <- effect + apply(effex, 1, sum)
+      effect <- effect + rowSums(effex)
       #
       # construct the corresponding function
       gd <- getglmdata(model)
@@ -293,7 +299,7 @@ parres <- function(model, covariate, ...,
         if(!all(d$offnames %in% colnames(mf))) 
           stop("Internal error: mismatch in term names in effectFun")
         moff <- mf[, d$offnames, drop=FALSE]
-        y <- apply(moff, 1, sum)
+        y <- rowSums(moff)
         return(y)
       }
     }
@@ -546,23 +552,21 @@ print.parres <- function(x, ...) {
            isoffset <- s$isoffset
            if(is.null(isoffset)) isoffset <- rep.int(FALSE, length(cancovs))
            ncc <- length(cancovs)
+           nfitted <- sum(!isoffset)
            noff <- sum(isoffset)
-           nother <- sum(!isoffset)
-           explain <-
-             paste(ngettext(ncc, "Fitted effect:", "Fitted effect: sum of"),
-                   if(noff == 0) {
-                     paste(paste(med, collapse=" and "),
-                           ngettext(ncc, "term", "terms"),
-                           commasep(dQuote(cancovs)))
-                   } else {
-                     paste(paste(med[med != "offset"], collapse=" and "),
-                           ngettext(nother, "term", "terms"),
-                           commasep(dQuote(cancovs[!isoffset])),
-                           "and offset",
-                           ngettext(noff, "term", "terms"),
-                           commasep(dQuote(cancovs[isoffset])))
-                   })
-           cat(paste(explain, "\n"))
+           explainfitted <- explainoff <- character(0)
+           if(noff > 0) 
+             explainoff <- paste("offset",
+                                 ngettext(noff, "term", "terms"),
+                                 commasep(dQuote(cancovs[isoffset])))
+           if(nfitted > 0)
+             explainfitted <- paste(
+               paste(med[med != "offset"], collapse=" and "),
+               ngettext(nfitted, "term", "terms"),
+               commasep(dQuote(cancovs[!isoffset])))
+           splat("Fitted effect: ",
+                 if(ncc > 1) "sum of" else NULL,
+                 paste(c(explainfitted, explainoff), collapse=" and "))
          },
          external={
            cat("Note: effect estimate not justified by delta method\n")
