@@ -1,7 +1,7 @@
 #
 #   plot.im.R
 #
-#  $Revision: 1.123 $   $Date: 2018/04/12 08:44:04 $
+#  $Revision: 1.127 $   $Date: 2019/03/08 10:13:03 $
 #
 #  Plotting code for pixel images
 #
@@ -133,10 +133,17 @@ plot.im <- local({
     return(y)
   }
 
-  Ticks <- function(usr, log=FALSE, nint=NULL, ...) {
-    #' same as axisTicks but accepts nint=NULL as if it were missing
-    if(!is.null(nint)) return(axisTicks(usr=usr, log=log, nint=nint, ...))
-    return(axisTicks(usr=usr, log=log, ...))
+  Ticks <- function(usr, log=FALSE, nint=NULL, ..., clip=TRUE) {
+    #' modification of grDevices::axisTicks
+    #'      constrains ticks to be inside the specified range if clip=TRUE
+    #'      accepts nint=NULL as if it were missing
+    z <- if(is.null(nint)) axisTicks(usr=usr, log=log, ...) else
+         axisTicks(usr=usr, log=log, nint=nint, ...) 
+    if(clip) {
+      zlimits <- if(log) 10^usr else usr
+      z <- z[inside.range(z, zlimits)]
+    }
+    return(z)
   }
 
   numericalRange <- function(x, zlim=NULL) {
@@ -325,23 +332,22 @@ plot.im <- local({
                col <- s$outputs
              } 
              trivial <- (diff(vrange) <= .Machine$double.eps)
-             if(!trivial) {
-               # ribbonvalues: domain of colour map (pixel values)
-               # ribbonrange: (min, max) of pixel values in image
-               # nominalrange: range of values shown on ribbon 
-               # nominalmarks: values shown on ribbon at tick marks
-               # ribbonticks: pixel values of tick marks 
-               # ribbonlabels: text displayed at tick marks
-               ribbonvalues <- seq(from=vrange[1], to=vrange[2],
-                                   length.out=ribn)
-               ribbonrange <- vrange
-               nominalrange <- Log(ribscale * Exp(ribbonrange))
-               nominalmarks <- user.ticks %orifnull% Ticks(nominalrange,
-                                                           log=do.log,
-                                                           nint=user.nint)
-               ribbonticks <- Log(nominalmarks/ribscale)
-               ribbonlabels <- paste(nominalmarks)
-             }
+             #' ribbonvalues: domain of colour map (pixel values)
+             #' ribbonrange: (min, max) of pixel values in image
+             #' nominalrange: range of values shown on ribbon 
+             #' nominalmarks: values shown on ribbon at tick marks
+             #' ribbonticks: pixel values of tick marks 
+             #' ribbonlabels: text displayed at tick marks
+             ribbonvalues <- if(trivial) vrange[1] else
+                             seq(from=vrange[1], to=vrange[2], length.out=ribn)
+             ribbonrange <- vrange
+             nominalrange <- Log(ribscale * Exp(ribbonrange))
+             nominalmarks <- if(trivial) nominalrange[1] else
+                             (user.ticks %orifnull% Ticks(nominalrange,
+                                                         log=do.log,
+                                                         nint=user.nint))
+             ribbonticks <- Log(nominalmarks/ribscale)
+             ribbonlabels <- paste(nominalmarks)
            },
            integer = {
              values <- as.vector(x$v)
@@ -362,8 +368,8 @@ plot.im <- local({
                }
                ribbonticks <- Log(nominalmarks/ribscale)
                ribbonlabels <- paste(nominalmarks)
-               if(!do.log && identical(all.equal(ribbonticks,
-                                                 vrange[1]:vrange[2]), TRUE)) {
+               if(!do.log && isTRUE(all.equal(ribbonticks,
+                                              vrange[1]:vrange[2]))) {
                  # each possible pixel value will appear in ribbon
                  ribbonvalues <- vrange[1]:vrange[2]
                  imagebreaks <- c(ribbonvalues - 0.5, vrange[2] + 0.5)
@@ -446,6 +452,7 @@ plot.im <- local({
       colourinfo <- list(breaks=imagebreaks, col=col)
     } else if(!is.null(colfun)) {
       ## Function colfun(n)
+      if(trivial) ncolours <- 1
       colourinfo <-
         if(is.null(imagebreaks)) list(col=colfun(ncolours)) else
         list(breaks=imagebreaks, col=colfun(length(imagebreaks) - 1L))
