@@ -36,7 +36,7 @@ local({
 #
 # Also test nnorient()
 #
-#   $Revision: 1.20 $  $Date: 2018/10/02 01:37:12 $
+#   $Revision: 1.22 $  $Date: 2019/02/21 03:01:30 $
 #
 
 require(spatstat)
@@ -93,6 +93,23 @@ local({
   if(any(nw5 != nw5P))
     stop("nnwhich.pp3(k=5) does not agree with pairdist")
 
+  Y <- runifpoint3(17)
+  a <- nncross(X,Y)
+  a <- nncross(X,Y, what="dist")
+  a <- nncross(X,Y, what="which")
+  a2 <- nncross(X,Y, k=2)
+  a2 <- nncross(X,Y, what="dist", k=2)
+  a2 <- nncross(X,Y, what="which", k=2)
+  iX <- 1:42
+  iZ <- 30:42
+  Z <- X[iZ]
+  b <- nncross(X, Z, iX=iX, iY=iZ)
+  b <- nncross(X, Z, iX=iX, iY=iZ, what="which")
+  b <- nncross(X, Z, iX=iX, iY=iZ, what="dist")
+  b2 <- nncross(X, Z, iX=iX, iY=iZ, k=2)
+  b2 <- nncross(X, Z, iX=iX, iY=iZ, what="which", k=2)
+  b2 <- nncross(X, Z, iX=iX, iY=iZ, what="dist", k=2)
+  
   # m dimensions
 
   X <- runifpointx(42, boxx(c(0,1),c(0,1),c(0,1),c(0,1)))
@@ -171,6 +188,16 @@ local({
   truewhich <- t(apply(crossdist(cells,flipcells), 1, order))[,1:4]
   if(any(calcwhich != truewhich))
     stop("nncross(k > 1) gives wrong answer")
+
+  #' cover some C code blocks
+  Z <- runifpoint(50)
+  X <- Z[1:30]
+  Y <- Z[20:50]
+  iX <- 1:30
+  iY <- 20:50
+  Ndw <- nncross(X,Y, iX, iY, k=3)
+  Nw  <- nncross(X,Y, iX, iY, k=3, what="which")
+  Nd  <- nncross(X,Y, iX, iY, k=3, what="dist")
   
   # test of agreement between nngrid.h and knngrid.h
   #    dimyx=23 (found by trial-and-error) ensures that there are no ties 
@@ -497,7 +524,7 @@ grep#
 #
 #   Plus assorted tricks
 #
-#   $Revision: 1.9 $  $Date: 2018/05/11 02:19:32 $
+#   $Revision: 1.10 $  $Date: 2019/02/10 07:00:38 $
 #
 require(spatstat)
 local({
@@ -564,13 +591,20 @@ local({
   parameters(ps)
   fitin(ps)
   predict(ps, type="cif")
+
+  ## (9) class 'plotppm'
+  fut <- ppm(amacrine ~ marks + polynom(x,y,2), Strauss(0.07))
+  p <- plot(fut, plot.it=FALSE)
+  print(p)
+  plot(p, how="contour")
+  plot(p, how="persp")
 })
 
 reset.spatstat.options()
 #'
 #'   tests/ppp.R
 #'
-#'   $Revision: 1.3 $ $Date: 2018/09/16 04:40:00 $
+#'   $Revision: 1.4 $ $Date: 2018/11/05 00:50:01 $
 #'
 #'  Untested cases in ppp() or associated code
 
@@ -587,13 +621,16 @@ local({
   D <- ppp(X$x, c(X$y[1:7], c(Inf, NA, NaN)), window=letterR,
            marks=data.frame(id=1:10, u=runif(10)))
 
-  #' test print/summary methods on these bad objects
+  #' test print/summary/plot methods on these bad objects
   print(A)
   print(B)
   print(D)
   summary(A)
   summary(B)
   summary(D)
+  plot(A)
+  plot(B)
+  plot(D)
   
   #' subset operator --- cases not covered elsewhere
   #'   subset index is a logical image
@@ -633,7 +670,7 @@ local({
 #
 # Test operations for ppx objects
 #
-#  $Revision: 1.4 $ $Date: 2018/04/30 09:28:15 $
+#  $Revision: 1.5 $ $Date: 2019/01/02 07:58:20 $
 #
 
 require(spatstat)
@@ -644,22 +681,33 @@ local({
   X <- ppx(data=df, coord.type=rep("s", 3), domain=box3())
   unique(X)
   duplicated(X)
+  anyDuplicated(X)
   multiplicity(X)
   print(X)
+  summary(X)
   plot(X)
   domain(X)
   unitname(X) <- c("metre", "metres")
   unitname(X)
-  
+
   #' subset operator
   X[integer(0)]
   Y <- X %mark% data.frame(a=df$x, b=1:4)
   Y[1:2]
   Y[FALSE]
+  marks(Y) <- as.data.frame(marks(Y))
+  Y[integer(0)]
+  Y[1:2]
+  Y[FALSE]
 
-  #' two dimensions
+  #' two dimensional
   A <- ppx(data=df[,1:2], coord.type=rep("s", 2), domain=square(1))
   plot(A)
+  B <- ppx(data=df[,1:2], coord.type=rep("s", 2), domain=NULL)
+  plot(B)
+  #' one dimensional
+  E <- ppx(data=data.frame(x=runif(10)))
+  plot(E)
   
   #' bug
   stopifnot(identical(unmark(chicago[1]),
@@ -670,13 +718,28 @@ local({
   V <- U %mark% 1
   V <- U %mark% factor("a")
 
+  #' simplify lower-dimensional patterns
+  X3 <- ppx(data=df, coord.type=rep("s", 3), domain=box3(), simplify=TRUE)
+  stopifnot(is.pp3(X3))
+  X2 <- ppx(data=df[,1:2], coord.type=rep("s", 2), domain=square(1), simplify=TRUE)
+  stopifnot(is.ppp(X2))
+
+  #' marks<-.ppx
+  M <- as.matrix(X)
+  marks(X) <- df[,1]
+  marks(X) <- df[,integer(0)]
+
+  #' trivial cases of random generators
+  B4 <- boxx(0:1, 0:1, 0:1, 0:1)
+  Z0 <- runifpointx(0, domain=B4, nsim=2)
+  Z1 <- runifpointx(1, domain=B4, nsim=2)
 })
 #
 # tests/prediction.R
 #
 # Things that might go wrong with predict()
 #
-#  $Revision: 1.8 $ $Date: 2018/11/01 13:20:23 $
+#  $Revision: 1.10 $ $Date: 2019/01/14 07:07:20 $
 #
 
 require(spatstat)
@@ -705,7 +768,11 @@ local({
     stop("new.coef mechanism is broken!")
   if(max(abs(pn-p0)) > 0.01)
     stop("new.coef mechanism gives wrong answer, for unnamed vectors")
-
+  #' adaptcoef     
+  a <- c(A=1,B=2,Z=42)
+  b <- c(B=41,A=0)
+  ab <- adaptcoef(a, b, drop=TRUE)
+  
   # tests of relrisk.ppm
   fut <- ppm(amacrine ~ x * marks)
   a <- relrisk(fut, control=2, relative=TRUE)
@@ -744,6 +811,15 @@ local({
   fut1 <- ppm(cells ~ 1, Strauss(0.1))
   effectfun(fut1, "x")
   effectfun(fut1, "y")
+
+  ## ppm with covariate values in data frame
+  X <- rpoispp(42)
+  Q <- quadscheme(X)
+  weirdfunction <- function(x,y){ 10 * x^2 + 5 * sin(10 * y) }
+  Zvalues <- weirdfunction(x.quad(Q), y.quad(Q))
+  fot <- ppm(Q ~ y + Z, data=data.frame(Z=Zvalues))
+  effectfun(fot, "y", Z=0)
+  effectfun(fot, "Z", y=0)
 })
 
 #
